import anndata
import numpy as np
from anndata import AnnData
from rpy2 import robjects
from rpy2.robjects import pandas2ri

from rpy2.robjects.conversion import localconverter
import pandas as pd
import pickle

from scipy.stats import pearsonr

import rectanglepy as rectanglepy
from rectanglepy.pp import RectangleSignatureResult
from rectanglepy.tl.deconvolution import deconvolution
from rectanglepy.pp.create_signature import build_rectangle_signatures
def rds_to_df(filename, is_df=False):
    r_file = filename
    robjects.r(f"df_to_load <- readRDS('{r_file}')")
    r_df = robjects.r["df_to_load"]

    # Convert R dataframe to pandas dataframe
    with localconverter(robjects.default_converter + pandas2ri.converter):
        p_df = robjects.conversion.rpy2py(r_df)
    if(is_df):
        p_df = pd.DataFrame(p_df)
        p_df.index = r_df.rownames
        p_df.columns = r_df.colnames

    return p_df

def brca(data_dir):
    sc_data = rds_to_df(data_dir / "brca_counts_no_cancer.rds", True)
    annotations =  rds_to_df(data_dir / "brca_annotations_no_cancer.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)
    sc_data, annotations = rectanglepy.pp.make_pseudo_bulk(sc_data, annotations, 10)
    annotations = pd.Series(annotations, index=sc_data.columns)

    return sc_data, annotations

def sig():
    import pickle
    rectangle_pickle = open('../../rectangle_data/tests/rectangle_signature_hao1.pkl', 'rb')
    signature = pickle.load(rectangle_pickle)
    bulk_finotello = rds_to_df('../../rectangle_data/tests/finotello_tpm.rds', True)
    decon_result = bulk_finotello.apply(lambda x:rectangle.tl.direct_deconvolute(signature.signature, x, signature.pseudo_signature), axis=0)
    return signature



def  create_sig_limma():

    sc_data = rds_to_df("../../rectangle_data/tests/hao1_matrix_counts.rds", True)
    bulk_data = rds_to_df("../../rectangle_data/tests/hoek_tpm.rds", True)
    genes = list(set(bulk_data.index) & set(sc_data.index))
    sc_data = sc_data.loc[genes].sort_index()
    bulk_data = bulk_data.loc[genes].sort_index().astype("double")
    #convert to sparse datafraxme
    annotations =  rds_to_df("../../rectangle_data/tests/hao1_celltype_annotations.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)
    signatures = rectangle.pp.build_rectangle_signatures(sc_data, annotations,0.022, 1.1)
    print(len(signatures.signature_genes))
    import pickle
    with open('rectangle_deseq_l1_p02_pval_fixed_low_condition_filtered_min8_fino', 'wb') as f:
        pickle.dump(signatures, f)

def  hoek_rectangle():

    sc_data = rds_to_df("../../rectangle_data/tests/hao1_matrix_counts.rds", True)
    sc_data = sc_data.loc[sc_data.sum(axis=1) > 10]
    bulks = rds_to_df("../../rectangle_data/tests/hoek_tpm.rds", True)
    annotations =  rds_to_df("../../rectangle_data/tests/hao1_celltype_annotations.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)

    genes = list(set(bulks.index) & set(sc_data.index))
    sc_data = sc_data.loc[genes].sort_index()
    bulks = bulks.loc[genes].sort_index()

    adata = AnnData(sc_data.T, obs=annotations.to_frame(name="cell_type"))
    signatures = build_rectangle_signatures(adata, "cell_type",optimize_cutoffs=True)
    bulks = AnnData(bulks.T, obs=bulks.columns.to_frame(name="bulk"))
    estimations = deconvolution(signatures, bulks)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_hoek_averaged.csv")
    genes = signatures.signature_genes
    cpm_sig = signatures.pseudobulk_sig_cpm
    cpm_sig = cpm_sig.loc[genes]
    cpm_sig.to_csv("../../rectangle_data/tests/hoek_sig_averaged.csv")


def salmon():

    salmon_bulks = pd.read_csv("../../rectangle_data/salmon.tsv", index_col=1, sep="\t")
    # drop column gene_id
    salmon_bulks = salmon_bulks.drop(columns=['gene_id'])
    with open('../../rectangle_data/tests/rectangle_sig_hoek', 'rb') as f:
        signatures = pickle.load(f)


    bulks = AnnData(salmon_bulks.T, obs=salmon_bulks.columns.to_frame(name="bulk"))
    estimations = deconvolution(signatures, bulks)

    estimations.to_csv("../../rectangle_data/tests/rectangle_result_salmon.csv")
def fino_rectangle():

    bulks_fino = rds_to_df("../../rectangle_data/tests/finotello_tpm.rds", True)
    bulks_hoek = rds_to_df("../../rectangle_data/tests/hoek_tpm.rds")



    sc_data = rds_to_df("../../rectangle_data/hao_complete/matrix_counts.rds", True)
    bulk_genes = list(set(bulks_fino.index))
    # only keep sc_data rows that are in bulk data
    #genes = list(set(bulks_fino.index) & set(sc_data.index))
    #sc_data = sc_data.loc[genes].sort_index()


    annotations = rds_to_df("../../rectangle_data/hao_complete/celltype_annotations.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)




    adata = AnnData(sc_data.T, obs=annotations.to_frame(name="cell_type"))
    # write adata to file
    adata.write("../../rectangle_data/tests/rectangle_adata.h5ad")
    results = rectanglepy.rectangle(adata, bulks_fino.T, consensus_runs=5, sample_size=1000)[0]
    results.to_csv("../../rectangle_data/tests/rectangle_result_fino_consensus.csv")

    signatures = build_rectangle_signatures(adata, "cell_type", bulks=bulks_fino.T)


    import pickle
    #with open('../../rectangle_data/tests/rectangle_sig_hao_clustered', 'rb') as f:
     #   signatures = pickle.load(f)
    # pickle

    with open('../../rectangle_data/tests/rectangle_sig_hao_clustered', 'wb') as f:
        pickle.dump(signatures, f)

    estimations = deconvolution(signatures, bulks_fino.T)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_fino.csv")


    estimations = deconvolution(signatures, bulks_hoek)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_hoek.csv")





    #bulks_kassanfra = AnnData(bulks_kassanfra.T, obs=bulks_kassanfra.columns.to_frame(name="bulk"))
    #estimations = deconvolution(signatures, bulks_kassanfra, False)
    #estimations.to_csv("../../rectangle_data/tests/rectangle_result_kassandra_hao.csv")


def subsample():

    bulks_fino = rds_to_df("../../rectangle_data/tests/finotello_tpm.rds", True)
    bulks_hoek = rds_to_df("../../rectangle_data/tests/hoek_tpm.rds")



    sc_data = rds_to_df("../../rectangle_data/matrix_subsampled_sub.rds", True)
    bulks_hoek = AnnData(bulks_hoek.T, obs=bulks_hoek.columns.to_frame(name="bulk"))
    bulks_fino = AnnData(bulks_fino.T, obs=bulks_fino.columns.to_frame(name="bulk"))


    annotations = rds_to_df("../../rectangle_data/celltype_annotations_sub.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)

    # Get the count of each annotation
    annotation_counts = annotations.value_counts()

    # Get the annotations with a count below 50


    adata = AnnData(sc_data.T, obs=annotations.to_frame(name="cell_type"))
    signatures = build_rectangle_signatures(adata, "cell_type")

    import pickle
    #with open('../../rectangle_data/tests/rectangle_sig_hao_clustered', 'rb') as f:
     #   signatures = pickle.load(f)
    # pickle

    with open('../../rectangle_data/tests/rectangle_sig_hao_clustered', 'wb') as f:
        pickle.dump(signatures, f)


    estimations = deconvolution(signatures, bulks_hoek)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_hoek.csv")



    estimations = deconvolution(signatures, bulks_fino)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_fino.csv")

    #bulks_kassanfra = AnnData(bulks_kassanfra.T, obs=bulks_kassanfra.columns.to_frame(name="bulk"))
    #estimations = deconvolution(signatures, bulks_kassanfra, False)
    #estimations.to_csv("../../rectangle_data/tests/rectangle_result_kassandra_hao.csv")








def create_sig_sapiens():

    sc_data = rds_to_df("../../rectangle_data/tests/sapiens_counts_dense.rds", True)
    annotations =  rds_to_df("../../rectangle_data/tests/sapiens_annotations.rds")
    annotations = pd.Series(annotations['cell_type'], index=annotations.index)

    annotations = annotations.iloc[::3]
    sc_data = sc_data.iloc[:,::3]
    signatures = rectangle.pp.build_rectangle_signatures(sc_data, annotations, 0.3, 1.5)
    import pickle
    with open('rectangle_deseq_sapiens_blood_l15_p3', 'wb') as f:
        pickle.dump(signatures, f)
def create_sig_mouse():

    sc_data = rds_to_df("../../rectangle_data/tests/tabula-muris/matrix_counts.rds", True)
    #convert to sparse dataframe
    annotations =  rds_to_df("../../rectangle_data/tests/tabula-muris/celltype_annotations.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)
    bulk_data = rds_to_df("../../rectangle_data/tests/chen_tpm.rds", True)
    genes = list(set(bulk_data.index) & set(sc_data.index))
    sc_data = sc_data.loc[genes].sort_index()
    signatures = rectangle.pp.build_rectangle_signatures(sc_data, annotations, 0.021, 1)
    import pickle
    with open('../../rectangle_data/tests/rectangle_deseq_mouse_l1_p021_min10_pept', 'wb') as f:
        pickle.dump(signatures, f)
def  peti_rectangle():
    sc_data = rds_to_df("../../rectangle_data/tests/tabula-muris/matrix_counts.rds", True)
    print(sc_data.info())
    #convert to sparse dataframe
    annotations =  rds_to_df("../../rectangle_data/tests/tabula-muris/celltype_annotations.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)
    bulk_data_peti = rds_to_df("../../rectangle_data/tests/petitprez_tpm.rds", True)
    bulk_data_chen = rds_to_df("../../rectangle_data/tests/chen_tpm.rds", True)
    bulk_genes = list(set(bulk_data_peti.index))

    adata = AnnData(sc_data.T, obs=annotations.to_frame(name="cell_type"))

    import pickle
    with open('../../rectangle_data/tests/rectangle_signatures_muris', 'rb') as f:
        signatures = pickle.load(f)
    signatures = build_rectangle_signatures(adata)
    # dump
    with open('../../rectangle_data/tests/rectangle_signatures_muris', 'wb') as f:
        pickle.dump(signatures, f)



    bulk_adata_chen = AnnData(bulk_data_chen.T, obs=bulk_data_chen.columns.to_frame(name="bulk"))
    estimations = deconvolution(signatures, bulk_adata_chen)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_chen.csv")

    bulk_adata_peti = AnnData(bulk_data_peti.T, obs=bulk_data_peti.columns.to_frame(name="bulk"))
    estimations = deconvolution(signatures, bulk_adata_peti)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_peti.csv")



def chen_rectangle():
    sc_data = rds_to_df("../../rectangle_data/tests/tabula-muris/matrix_counts.rds", True)
    # convert to sparse dataframe
    annotations = rds_to_df("../../rectangle_data/tests/tabula-muris/celltype_annotations.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)
    bulk_data = rds_to_df("../../rectangle_data/tests/chen_tpm.rds", True)
    genes = list(set(bulk_data.index) & set(sc_data.index))
    sc_data = sc_data.loc[genes].sort_index()
    bulk_data = bulk_data.loc[genes].sort_index()
    adata = AnnData(sc_data.T, obs=annotations.to_frame(name="cell_type"))
    signatures = build_rectangle_signatures(adata, "cell_type",optimize_cutoffs=True)
    bulk_adata = AnnData(bulk_data.T, obs=bulk_data.columns.to_frame(name="bulk"))
    estimations = deconvolution(signatures, bulk_adata)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_chen_averaged.csv")
    genes = signatures.signature_genes
    cpm_sig = signatures.pseudobulk_sig_cpm
    cpm_sig = cpm_sig.loc[genes]
    cpm_sig.to_csv("../../rectangle_data/tests/chen_sig_averaged.csv")
def brca_limma():
    sc_data = rds_to_df("../../rectangle_data/tests/brca_counts_no_cancer.rds", True)
    bulk_data = rds_to_df("../../rectangle_data/tests/brca_bulk_no_cancer.rds", True)
    genes = list(set(bulk_data.index) & set(sc_data.index))
    sc_data = sc_data.loc[genes].sort_index()
    annotations =  rds_to_df("../../rectangle_data/tests/brca_annotations_no_cancer.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)
    signatures = rectangle.pp.build_rectangle_signatures(sc_data, annotations, 0.018,1.1)

    import pickle
    with open('../../rectangle_data/tests/rectangle_deseq_brca_p018_l11', 'wb') as f:
        pickle.dump(signatures, f)
def brca_limma_rectangle():
    gld = rds_to_df("brca_gld_no_cancer.rds", True)
    gld.to_csv("brca_gld_no_cancer.csv")

    sc_data = rds_to_df("../../rectangle_data/tests/brca_counts_no_cancer.rds", True)
    bulk_data = rds_to_df("../../rectangle_data/tests/brca_bulk_no_cancer.rds", True)
    genes = list(set(bulk_data.index) & set(sc_data.index))
    sc_data = sc_data.loc[genes].sort_index()
    annotations = rds_to_df("../../rectangle_data/tests/brca_annotations_no_cancer.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)


    adata = AnnData(sc_data.T, obs=annotations.to_frame(name="cell_type"))
    signatures = build_rectangle_signatures(adata, "cell_type")
    bulk_adata = AnnData(bulk_data.T, obs=bulk_data.columns.to_frame(name="bulk"))
    estimations = deconvolution(signatures, bulk_adata)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_brca.csv")
    genes = signatures.signature_genes
    cpm_sig = signatures.pseudobulk_sig_cpm
    cpm_sig = cpm_sig.loc[genes]
    cpm_sig.to_csv("../../rectangle_data/tests/brca_sig_averaged.csv")



def dream_rectangle():
    sc_data = rds_to_df("../../rectangle_data/tests/brca_counts_no_cancer.rds", True)
    bulk_data = rds_to_df("../../rectangle_data/tests/brca_bulk_no_cancer.rds", True)
    annotations = rds_to_df("../../rectangle_data/tests/brca_annotations_no_cancer.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)
    estimations = rectangle(sc_data, annotations, bulk_data)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_dream.csv")

def r_to_pandas(df_r):
    with localconverter(robjects.default_converter + pandas2ri.converter):
        return robjects.conversion.get_conversion().rpy2py(df_r)


def dream():
    # open with scipy.io

    annotations = list(pd.read_csv("../../rectangle_data/tests/wu_minor_annotations_renamed.csv", header=None)[0])
    annotations3 = list(pd.read_csv("../../rectangle_data/tests/wu_subset_annotations.csv", header=None)[0])
    i = 0
    for a in annotations3:
        if a == 'T_cells_c2_CD4+_T-regs_FOXP3':
            annotations[i] = 'Tregs'
        i = i + 1
    bulk_data = pd.read_csv("../../rectangle_data/tests/GSE199324_symbol_tpm.csv", index_col=0)
    bulk_data = bulk_data.iloc[:, 21:]
    sc_data = rds_to_df("../../rectangle_data/tests/wu_major_counts.rds", True)
    genes = list(set(bulk_data.index) & set(sc_data.index))
    sc_data = sc_data.loc[genes].sort_index()
    annotations = pd.Series(annotations, index=sc_data.columns)
    annotations.to_csv("../../rectangle_data/tests/wu_minor_processed.csv")
    sc_data.to_pickle("../../rectangle_data/tests/wu_major_counts.pkl")
    bulk_data.to_csv("../../rectangle_data/tests/dream_tpm.csv")
    adata = AnnData(sc_data.T, obs=annotations.to_frame(name="cell_type"))
    import pickle
    with open('../../rectangle_data/tests/dream', 'rb') as f:
        signatures = pickle.load(f)
    signatures = build_rectangle_signatures(adata, "cell_type",optimize_cutoffs=True)
    import pickle
    with open('../../rectangle_data/tests/dream', 'wb') as f:
        pickle.dump(signatures, f)
    bulk_adata = AnnData(bulk_data.T, obs=bulk_data.columns.to_frame(name="bulk"))
    estimations = deconvolution(signatures, bulk_adata, False)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_dream_major.csv")
    genes = signatures.signature_genes
    cpm_sig = signatures.pseudobulk_sig_cpm
    cpm_sig = cpm_sig.loc[genes]
    cpm_sig.to_csv("../../rectangle_data/tests/dream_sig_major.csv")





def kassandra():
    genes = pd.read_csv("../../rectangle_data/tests/tabula_sapiens_hgnc.csv").x
    sc_data = pd.read_pickle("../../rectangle_data/tests/tabula_sapiens_counts.pkl")
    #sc_data = rds_to_df("../../rectangle_data/tests/tabula_sapiens_counts.rds", True)
    bulk_data = pd.read_csv("../../rectangle_data/tests/blood_expr.tsv", index_col=0, sep="\t")
    annotations =  pd.read_csv("../../rectangle_data/tests/tabula_sapiens_annotations.csv", index_col=0)

    annotations = annotations['0']
    annotations = pd.Series(annotations, index=sc_data.columns)

#    sc_data['hgnc'] = list(genes)
    # remove nan
 #   sc_data = sc_data.loc[~sc_data['hgnc'].isna()]
    # remove duplicates
  #  sc_data = sc_data.loc[~sc_data['hgnc'].duplicated()]

   # sc_data = sc_data.set_index('hgnc')

    adata = AnnData(sc_data.T, obs=annotations.to_frame(name="cell_type"))
    signatures = build_rectangle_signatures(adata, "cell_type",optimize_cutoffs=True)
    bulk_adata = AnnData(bulk_data.T, obs=bulk_data.columns.to_frame(name="bulk"))
    estimations = deconvolution(signatures, bulk_adata)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_kassandra_sapiens.csv")
    genes = signatures.signature_genes
    cpm_sig = signatures.pseudobulk_sig_cpm
    cpm_sig = cpm_sig.loc[genes]
    cpm_sig.to_csv("../../rectangle_data/tests/kassandra_sig.csv")

def kassandra_full():
    genes = pd.read_csv("../../rectangle_data/tests/tabula_sapiens_hgnc.csv").x
    sc_data = rds_to_df("../../rectangle_data/tests/tabula_sapiens_counts.rds", True)
    bulk_data = pd.read_csv("../../rectangle_data/tests/blood_expr.tsv", index_col=0, sep="\t")
    annotations =  rds_to_df("../../rectangle_data/tests/tabula_sapiens_annotations.rds")

    annotations = pd.Series(list(annotations.iloc[:,0]), index=sc_data.columns)

    sc_data['hgnc'] = list(genes)
    # remove nan
    sc_data = sc_data.loc[~sc_data['hgnc'].isna()]
    # remove duplicates
    sc_data = sc_data.loc[~sc_data['hgnc'].duplicated()]

    sc_data = sc_data.set_index('hgnc')


    sc_data_hao = rds_to_df("../../rectangle_data/tests/hao1_matrix_counts.rds", True)
    annotations_hao =  rds_to_df("../../rectangle_data/tests/hao1_celltype_annotations.rds")
    annotations_hao = pd.Series(annotations_hao, index=sc_data_hao.columns)
    # remove  monocytes
    sc_data_hao = sc_data_hao.loc[:, annotations_hao != 'Monocytes']
    annotations_hao = annotations_hao.loc[annotations_hao != 'Monocytes']

    sc_data_hao = sc_data_hao.loc[:, annotations_hao != 'B cells']
    annotations_hao = annotations_hao.loc[annotations_hao != 'B cells']

    sc_data_hao = sc_data_hao.loc[:, annotations_hao != 'Plasma cells']
    annotations_hao = annotations_hao.loc[annotations_hao != 'Plasma cells']

    sc_data_hao = sc_data_hao.loc[:, annotations_hao != 'Platelet']
    annotations_hao = annotations_hao.loc[annotations_hao != 'Platelet']

    sc_data_hao = sc_data_hao.loc[:, annotations_hao != 'T cells CD4 conv']
    annotations_hao = annotations_hao.loc[annotations_hao != 'T cells CD4 conv']

    sc_data_hao = sc_data_hao.loc[:, annotations_hao != 'T cells CD8']
    annotations_hao = annotations_hao.loc[annotations_hao != 'T cells CD8']

    sc_data_hao = sc_data_hao.loc[:, annotations_hao != 'pDC']
    annotations_hao = annotations_hao.loc[annotations_hao != 'pDC']

    sc_data_hao = sc_data_hao.loc[:, annotations_hao != 'mDC']
    annotations_hao = annotations_hao.loc[annotations_hao != 'mDC']


    annotations = annotations._append(annotations_hao)
    sc_data = sc_data.join(sc_data_hao, how='inner')
    adata = AnnData(sc_data.T, obs=annotations.to_frame(name="cell_type"))
    signatures = build_rectangle_signatures(adata, "cell_type",optimize_cutoffs=True)
    bulk_adata = AnnData(bulk_data.T, obs=bulk_data.columns.to_frame(name="bulk"))
    estimations = deconvolution(signatures, bulk_adata)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_kassandra_mixed.csv")
    genes = signatures.signature_genes
    cpm_sig = signatures.pseudobulk_sig_cpm
    cpm_sig = cpm_sig.loc[genes]
    cpm_sig.to_csv("../../rectangle_data/tests/kassandra_sig.csv")

def luca():
    luad_bulk = rds_to_df("../../rectangle_data/luad.rds", True)
    lusc_bulk = rds_to_df("../../rectangle_data/lusc.rds", True)
    # only every 10h bulk
    luad_bulk = luad_bulk.iloc[:, ::10]
    lusc_bulk = lusc_bulk.iloc[:, ::10]

    luca_sd = anndata.read_h5ad("../../rectangle_data/luca_lambrecht.h5ad")
    luca_sd = luca_sd[luca_sd.obs["cell_type_predicted"] != "Epithelial cell malignant"]
    # change var_names to  var['feature_name']
    luca_sd.var_names = luca_sd.var['feature_name']
    import pickle
   # with open('../../rectangle_data/tests/rectangle_sig_luca_no_tumor', 'rb') as f:
        #signatures = pickle.load(f)
    signatures = build_rectangle_signatures(luca_sd, raw=True, cell_type_col="cell_type_predicted")

    with open('../../rectangle_data/tests/rectangle_sig_luca_no_tumor', 'wb') as f:
        pickle.dump(signatures, f)
    bulk_adata = AnnData(luad_bulk.T, obs=luad_bulk.columns.to_frame(name="bulk"))
    estimations = deconvolution(signatures, bulk_adata, n_cpus=8)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_luad_10_no_tumor.csv")

    bulk_adata = AnnData(lusc_bulk.T, obs=lusc_bulk.columns.to_frame(name="bulk"))
    estimations = deconvolution(signatures, bulk_adata, n_cpus=8)
    estimations.to_csv("../../rectangle_data/tests/rectangle_result_lusc_10_no_tumor.csv")





def convert_deseq():
    rectangle_pickle = open('../../rectangle_data/tests/deseq.pickle', 'rb')
    rectangle_deseq_dict = pickle.load(rectangle_pickle)
    # rename dict keys replacing spaces with dots
    rectangle_deseq_dict = {k.replace(" ", "."): v for k, v in rectangle_deseq_dict.items()}
    # convert  all dict values to r
    for key, value in rectangle_deseq_dict.items():
        rectangle_deseq_dict[key] = rectangle.pp.pandas_to_r(value)

    import rpy2.robjects as robjects
    r_dict = robjects.ListVector(rectangle_deseq_dict)

    robjects.r.assign("my_r_object", r_dict)
    robjects.r("save(my_r_object, file='../../rectangle_data/tests/my_r_object.RData')")
def open_sig():
    import pickle
    rectangle_pickle = open('../../rectangle_data/tests/rectangle_limma_sig_05_25_correction_new_bias', 'rb')
    signature = pickle.load(rectangle_pickle)
    sparse_pickle = open('../../rectangle_data/tests/rectangle_limma_sig_05_25_correction_refactored_bias', 'rb')
    sparse_qp = pickle.load(sparse_pickle)
    return signature



def signature_creation_benchmark():
    import time
    sc_data = rds_to_df("../../rectangle_data/tests/hao1_matrix_counts.rds", True)
    sc_data2 = rds_to_df("../../rectangle_data/tests/hao-sampled-2/matrix_counts.rds", True)
    sc_data3 = rds_to_df("../../rectangle_data/tests/hao-sampled-3/matrix_counts.rds", True)
    sc_data4 = rds_to_df("../../rectangle_data/tests/hao-sampled-4/matrix_counts.rds", True)

    annotations = rds_to_df("../../rectangle_data/tests/hao1_celltype_annotations.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)

    annotations2 = rds_to_df("../../rectangle_data/tests/hao-sampled-2/celltype_annotations.rds")
    annotations2 = pd.Series(annotations2, index=sc_data2.columns)

    annotations3 = rds_to_df("../../rectangle_data/tests/hao-sampled-3/celltype_annotations.rds")
    annotations3 = pd.Series(annotations3, index=sc_data3.columns)

    annotations4 = rds_to_df("../../rectangle_data/tests/hao-sampled-4/celltype_annotations.rds")
    annotations4 = pd.Series(annotations4, index=sc_data4.columns)

    #combine both datasets
    sc_data = sc_data.join(sc_data2, how='inner', lsuffix='_1', rsuffix='_2')
    sc_data2 = None
    sc_data = sc_data.join(sc_data3, how='inner', rsuffix='_3')
    sc_data3 = None
    sc_data = sc_data.join(sc_data4, how='inner', rsuffix='_4')
    sc_data4 = None

    annotations = annotations._append(annotations2)
    annotations = annotations._append(annotations3)
    annotations = annotations._append(annotations4)




    # only take 20.000 columns
    sc_data = sc_data.iloc[:, :40000]
    annotations = annotations.iloc[:40000]

    # repeat columns 2 times to get 80.000 columns
    sc_data = sc_data.join(sc_data, how='inner', lsuffix='_1', rsuffix='_2')
    annotations = annotations._append(annotations)

    annotations.index = sc_data.columns

    print('length of annotations: ' + str(len(annotations)))
    adata = AnnData(sc_data.T, obs=annotations.to_frame(name="cell_type"))
    sc_data = None
    times = []
    average = 0
    min = 0
    max = 0
    for i in range(1):
        start = time.time()
        signatures = build_rectangle_signatures(adata, "cell_type",optimize_cutoffs=True)
        end = time.time()
        times.append(end - start)
        if i == 0:
            min = end - start
            max = end - start
        if end - start < min:
            min = end - start
        if end - start > max:
            max = end - start
        average = average + (end - start)
    average = average / 10
    print("min: " + str(min))
    print("max: " + str(max))
    print("average: " + str(average))



def deconvolution_benchmark():
    import time
    sc_data = rds_to_df("../../rectangle_data/tests/hao1_matrix_counts.rds", True)
    annotations = rds_to_df("../../rectangle_data/tests/hao1_celltype_annotations.rds")
    annotations = pd.Series(annotations, index=sc_data.columns)
    bulks = rds_to_df("../../rectangle_data/tests/finotello_tpm.rds", True)


    adata = AnnData(sc_data.T, obs=annotations.to_frame(name="cell_type"))
    signatures = build_rectangle_signatures(adata, "cell_type",optimize_cutoffs=True)
    bulk_adata = AnnData(bulks.T, obs=bulks.columns.to_frame(name="bulk"))
    times = []
    average = 0
    min = 0
    max = 0
    for i in range(10):
        start = time.time()
        estimations = deconvolution(signatures, bulk_adata)
        end = time.time()
        times.append(end - start)
        if i == 0:
            min = end - start
            max = end - start
        if end - start < min:
            min = end - start
        if end - start > max:
            max = end - start
        average = average + (end - start)
    average = average / 10
    print("min: " + str(min))
    print("max: " + str(max))
    print("average: " + str(average))


def consensus_solution():
    from sklearn.metrics import mean_squared_error

    mirrordb = ['0.01', '0.02', '0.05', '0.1', '0.2', '0.4', '100', '500', '1000', '1500', '2000', '2500']
    mirror_solutions = []
    even_solutions = {}
    # Initialize lists for each part of RectangleSignatureResult

    for m in mirrordb:
        for name in ['Hoek', 'Finotello']:
            fractions = []
            true_df = None
            for i in range(1, 11):
                foldername = f"../../rectangle_data/results_downsample/rectangle_hao-complete_counts_{name.lower()}_tpm_ct{m}_rep{i}"
                # open signatue_result.pkl
                r_file = foldername + "/results_metric.rds"
                robjects.r(f"df_to_load <- readRDS('{r_file}')")
                r_df = robjects.r["df_to_load"]
                # Convert R dataframe to pandas dataframe
                with localconverter(robjects.default_converter + pandas2ri.converter):
                    p_df = robjects.conversion.rpy2py(r_df[5])
                    true_df = robjects.conversion.rpy2py(r_df[4])
                fractions.append(p_df)


            # Calculate the median for fractions
            median_fracs = pd.concat(fractions).groupby(level=0).median()
            if name == 'Hoek':
                median_fracs = median_fracs[['T cell', 'Monocytes', 'B cells', 'mDC', 'NK cells']]
            if name == 'Finotello':

                median_fracs = median_fracs[['T cells CD4 conv', 'T cells CD8', 'NK cells', 'B cells','Tregs', 'mDC', 'Monocytes']]
            #sort dfs indices and columns
            true_df = true_df.T
            true_df = true_df.sort_index()
            median_fracs = median_fracs.sort_index()
            true_df = true_df[median_fracs.columns]
            common_indices = true_df.index.isin(median_fracs.index)
            true_df = true_df[common_indices]
            correlation =  true_df.corrwith(median_fracs, method='pearson')


            true_melted = true_df.melt()
            median_melted = median_fracs.melt()
            corr = pearsonr(median_melted['value'], true_melted['value'])[0]
            correlation['all'] = corr
            print(corr)


            # Calculate RMSE
            rmse_values = {}
            for col in true_df.columns:
                # Calculate RMSE for each column pair
                rmse = np.sqrt(((true_df[col] - median_fracs[col]) ** 2).mean())
                rmse_values[col] = rmse
            rmse_values['all'] = np.sqrt(np.mean((median_melted['value'] - true_melted['value']) ** 2))
            rmse_values = pd.Series(rmse_values)

            # Calculate the mean for fractions

            df = pd.DataFrame(
                {'cor': correlation,
                 'RMSE': rmse_values,
                 "cell_type": rmse_values.index,
                 "bulk_ds": name,
                 "ct": str(m),
                 "method": "Rectangle"},)
            mirror_solutions = mirror_solutions + [df]

    mirror_df = pd.concat(mirror_solutions)
    mirror_df.to_csv("../../rectangle_data/tests/consensus.csv", index=False)

def covid():
    sc_adata = anndata.read_h5ad("../../rectangle_data/covid_portal_210320_non_covid.h5ad")
    sc_adata = sc_adata[sc_adata.obs["initial_clustering"] != "MAIT"]
    sc_adata = sc_adata[sc_adata.obs["initial_clustering"] != "gdT"]
    sc_adata = sc_adata[sc_adata.obs["initial_clustering"] != "Lymph_prolif"]
    sc_adata = sc_adata[sc_adata.obs["initial_clustering"] != "Mono_prolif"]
    sc_adata = sc_adata[sc_adata.obs["initial_clustering"] != "CD14"]
    #Merge CD16 and CD14 into "monocytes"
    #sc_adata.obs["initial_clustering"] = sc_adata.obs["initial_clustering"].replace("CD16", "Monocytes")
    #sc_adata.obs["initial_clustering"] = sc_adata.obs["initial_clustering"].replace("CD14", "Monocytes")
    sc_adata.obs["initial_clustering"] = sc_adata.obs["initial_clustering"].replace("NK_16hi", "NK cells")
    sc_adata.obs["initial_clustering"] = sc_adata.obs["initial_clustering"].replace("NK_56hi", "NK cells")

    bulks_fino = rds_to_df("../../rectangle_data/tests/finotello_tpm.rds", True)
    bulks_hoek = rds_to_df("../../rectangle_data/tests/hoek_tpm.rds")

    results_hoek = rectanglepy.rectangle(sc_adata, bulks_hoek.T, sample_size=1500, subsample=True,cell_type_col="initial_clustering", layer="raw", consensus_runs=1)
    results_hoek[0].to_csv("../../rectangle_data/tests/rectangle_result_hoek_covid_1500.csv")

    results_fino = rectanglepy.rectangle(sc_adata, bulks_fino.T, sample_size=1500, subsample=True, cell_type_col="initial_clustering", layer="raw", consensus_runs=1)
    results_fino[0].to_csv("../../rectangle_data/tests/rectangle_result_fino_covid_1500.csv")

    x = 10

















import scipy.io
"""
annotations = list(pd.read_csv("../../rectangle_data/tests/wu_minor_annotations_renamed.csv", header=None)[0])
annotations2 = list(pd.read_csv("../../rectangle_data/tests/wu_minor_annotations.csv", header=None)[0])
annotations3 = list(pd.read_csv("../../rectangle_data/tests/wu_subset_annotations.csv", header=None)[0])
annotations4 = list(pd.read_csv("../../rectangle_data/tests/wu_major_annotations.csv", header=None)[0])
i = 0
for a in annotations3:
    if a == 'T_cells_c2_CD4+_T-regs_FOXP3':
        annotations[i] = 'Tregs'
    i = i + 1

# count annotations
from collections import Counter
x = Counter(annotations)
y  = Counter(annotations2)
z = Counter(annotations3)

dict_annot = {'major': annotations4, 'minor': annotations2, 'relabeled_minor':annotations ,'subset': annotations3}
df = pd.DataFrame(dict_annot)
"""

#deconvolution_benchmark()
#signature_creation_benchmark()
#brca_limma_rectangle()

#dream()
#kassandra_full()
#peti_rectangle()
#subsample()
fino_rectangle()
#salmon()
#luca()
#sc_counts,annotations, bulks,  = rectanglepy.pp.load_tutorial_sc_data()

#adata = AnnData(sc_counts.T, obs=annotations.to_frame(name="cell_type"))
#signatures = build_rectangle_signatures(adata, "cell_type")

#print(signatures)




#chen_rectangle()



covid()




#hoek_rectangle()




















